// Copyright (C) 2000 - 2002 Hewlett-Packard Company
//
// This program is free software; you can redistribute it and/or modify it
// under the term of the GNU Lesser General Public License as published by the
// Free Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
// for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program; if not, write to the Free Software Foundation,
// Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// _________________

// @(#) $Revision: 1.1.1.1 $ $Source: /project/eecs/db/cvsroot/Judy-am-TCQ/src/JudyCommon/JudyMalloc.c,v $
// ************************************************************************ //
//                    JUDY - Memory Allocater                             //
//                              -by-					  //
//		         Douglas L. Baskins				  //
//			  Hewlett Packard				  //
//                        Fort Collins, Co				  //
//                         (970) 229-2027				  //
//									  //
// ************************************************************************ //

// JUDY INCLUDE FILES

#ifdef JU_FLAVOR_COV
#pragma C-Cover off	// exclude inline functions in public header files.
#endif
#include <stdlib.h>	// for malloc().
#ifdef JU_FLAVOR_COV
#pragma C-Cover on
#endif

#include "JudyCommon/JudyMalloc.h"
#include "JudyCommon/JudyPrivate.h"

// #define JUDY_INS_TEST 1
#ifdef JUDY_INS_TEST
Word_t malloccnt[550];	// TBD: Correct size?
Word_t freecnt  [550];	// TBD: Correct size?
#endif

JudyMallocHelperType JudyMallocHelper = malloc;
JudyFreeHelperType   JudyFreeHelper   = free;


// ****************************************************************************
// J U D Y   M A L L O C
//
// Allocate RAM.  This is the single location in Judy code that calls
// malloc(3C).  Note:  JPM accounting occurs at a higher level.

FUNCTION Word_t JudyMalloc(
	Word_t Words)
{
	Word_t Addr;

#ifdef JUDY_INS_TEST
        malloccnt[Words]++;
#endif
	Addr = (Word_t) ((* JudyMallocHelper)(Words * sizeof(Word_t)));
	return(Addr);

} // JudyMalloc()


// ****************************************************************************
// J U D Y   F R E E

FUNCTION void JudyFree(
	void * PWord,
	Word_t Words)
{
#ifdef JUDY_INS_TEST
        freecnt[Words]++;
#endif
	(* JudyFreeHelper)(PWord);

} // JudyFree()

// ****************************************************************************
// J U D Y   M A L L O C
//
// Higher-level "wrapper" for allocating objects that need not be in RAM,
// although at this time they are in fact only in RAM.  Later we hope that some
// entire subtrees (at a JPM or branch) can be "virtual", so their allocations
// and frees should go through this level.

FUNCTION Word_t JudyMallocVirtual(
	Word_t Words)
{
	return(JudyMalloc(Words));

} // JudyMallocVirtual()


// ****************************************************************************
// J U D Y   F R E E

FUNCTION void JudyFreeVirtual(
	void * PWord,
	Word_t Words)
{
        JudyFree(PWord, Words);

} // JudyFreeVirtual()

// ****************************************************************************
// J U D Y   S E T   H E L P E R S
//
// Sets up the Malloc and Free Helper functions for Judy
//
void JudySetMemoryHelpers(JudyMallocHelperType m, JudyFreeHelperType f)
{
  JudyMallocHelper = m;
  JudyFreeHelper = f;
}
